@@ -1,11 +1,15 @@
 from __future__ import annotations
+
 import copy
 import re
-from typing import Union, Callable, Optional
-from collections import OrderedDict, Counter
+from collections import Counter, OrderedDict
+from typing import Callable, Optional, Union
 
-import numpy as np
 import elementy
+import numpy as np
+
+from .prototype import Prototype
+from .prototypes import get_prototype
 
 
 class Alloy:
@@ -16,9 +20,12 @@ class Alloy:
     Attributes
     ----------
 
-    composition : dict
+    composition
         Dictionary matching element symbols to atomic percentages.
-    constraints : dict
+    structure
+        String referring to a crystal structure prototype which defines atomic
+        percentages.
+    constraints
         Dictionary of constraints to follow when adjusting atomic  percentages.
 
     """
@@ -63,13 +70,25 @@ class Alloy:
     def __init__(
         self,
         composition: Union[str, dict, Alloy],
-        constraints: Union[dict, None] = None,
+        structure: Optional[Union[str, Prototype]] = None,
+        constraints: Optional[dict] = None,
         rescale: bool = True,
     ):
 
-        self.composition = parse_composition(composition)
+        self.original_composition, composition_structure = parse_composition(
+            composition
+        )
+        self.original_elements = list(self.original_composition.keys())
+        self.composition = filter_order_composition(self.original_composition)
+
         if self.composition is None:
             raise Exception("Invalid composition:", composition)
+        if structure is None and composition_structure is not None:
+            structure = composition_structure
+
+        self._structure = None
+        if structure is not None:
+            self.structure = structure
 
         self.constraints = None
 
@@ -129,6 +148,59 @@ class Alloy:
         if not hasattr(self, "rescaling"):
             self.rescale()
 
+    @property
+    def structure(self) -> str:
+        return self._structure
+
+    @structure.setter
+    def structure(self, structure):
+
+        if isinstance(structure, str):
+            structure = copy.deepcopy(get_prototype(structure))
+        self._structure = structure
+
+        if not all(
+            value == list(self.original_composition.values())[0]
+            for value in self.original_composition.values()
+        ):
+            structure_elements_ordered = sorted(
+                self.structure.composition,
+                key=lambda x: self.structure.composition[x],
+                reverse=True,
+            )
+
+            composition_elements_ordered = [
+                list(self.original_composition.keys()).index(e)
+                for e in self.original_composition
+            ]
+
+            index_translation = {
+                i: min([j, max(composition_elements_ordered)])
+                for i, j in zip(
+                    composition_elements_ordered,
+                    structure_elements_ordered,
+                )
+            }
+
+            for b in self.structure.basis:
+                if b["element"] in index_translation:
+                    b["element"] = index_translation[b["element"]]
+
+        structure_composition = {}
+        for structure_element_index in self.structure.elements:
+            if structure_element_index >= len(self.elements):
+                element_index = len(self.elements) - 1
+            else:
+                element_index = structure_element_index
+
+            if self.elements[element_index] not in structure_composition:
+                structure_composition[self.elements[element_index]] = 0
+            structure_composition[
+                self.elements[element_index]
+            ] += self.structure.composition[element_index]
+
+        self.composition = structure_composition
+
     @property
     def elements(self) -> list:
         """List of elements in the alloy.
@@ -137,6 +209,25 @@ class Alloy:
         """
         return list(self.composition.keys())
 
+    @property
+    def elements_structure_order(self) -> list:
+        """List of elements in the alloy in the order of the structure basis.
+
+        :group: alloy
+        """
+        if self.structure is not None:
+            unique_elements = []
+            for b in self.structure.original_basis:
+                if b["element"] < len(self.original_elements) and (
+                    self.original_elements[b["element"]] not in unique_elements
+                ):
+                    unique_elements.append(
+                        self.original_elements[b["element"]]
+                    )
+            return unique_elements
+        else:
+            return self.elements
+
     @property
     def num_elements(self) -> int:
         """Number of elements in the alloy.
@@ -317,8 +408,21 @@ class Alloy:
         delattr(self, "rescaling")
 
     def reorder_composition(self):
+        ordered_elements = self.elements
+        if self.constraints is not None:
+            ordered_elements = sorted(
+                self.composition,
+                key=lambda x: (
+                    self.constraints["percentages"][x]["precedence"]
+                    if x in self.constraints["percentages"]
+                    else 0
+                ),
+                reverse=True,
+            )
         ordered_elements = sorted(
-            self.composition, key=self.composition.get, reverse=True
+            ordered_elements,
+            key=self.composition.get,
+            reverse=True,
         )
         for element in ordered_elements:
             self.composition.move_to_end(element)
@@ -784,6 +888,9 @@ class Alloy:
 
             composition_str += element + percentage_str
 
+        if self.structure is not None:
+            composition_str += "[" + self.structure.name + "]"
+
         return composition_str
 
     def to_pretty_string(self) -> str:
@@ -991,9 +1098,9 @@ def parse_composition(composition: Union[str, dict, Alloy]) -> dict:
     if isinstance(composition, str):
         return parse_composition_string(composition)
     elif isinstance(composition, dict):
-        return parse_composition_dict(composition)
+        return parse_composition_dict(composition), None
     elif isinstance(composition, Alloy):
-        return composition.composition
+        return composition.composition, None
 
 
 def parse_composition_string(composition_string: str) -> dict:
@@ -1005,24 +1112,19 @@ def parse_composition_string(composition_string: str) -> dict:
     if len(composition_string) == 0:
         return None
 
-    if "(" in composition_string:
+    if "[" in composition_string:
+        structure = composition_string.split("[")[1].split("]")[0]
+        composition_string = composition_string.split("[")[0]
+    else:
+        structure = None
 
-        sub_composition_percentage = (
-            float(
-                re.split(r"(\d+(?:\.\d+)?)", composition_string.split(")")[1])[
-                    1
-                ]
-            )
-            / 100.0
-        )
+    if "(" in composition_string:
 
         sub_composition_string = composition_string.split(")")[0].split("(")[1]
 
         sub_composition = parse_composition_string_block(
             sub_composition_string
         )
-        for element in sub_composition:
-            sub_composition[element] *= sub_composition_percentage
 
         pre_sub_composition_string = composition_string.split(")")[0].split(
             "("
@@ -1037,16 +1139,48 @@ def parse_composition_string(composition_string: str) -> dict:
         if re.search("[a-zA-Z]", post_sub_composition_string) is not None:
             remaining_composition_string += post_sub_composition_string
 
+        split_composition = re.findall(
+            r"[A-Z][^A-Z]*", remaining_composition_string
+        )
+
+        composition_sum = 0
+        for element_percentage in split_composition:
+            split_element_percentage = re.split(
+                r"(\d+(?:\.\d+)?)", element_percentage
+            )
+            if len(split_element_percentage) > 1:
+                composition_sum += float(split_element_percentage[1])
+        if composition_sum == 0:
+            composition_sum = 1
+
+        sub_composition_percentage = float(
+            re.split(r"(\d+(?:\.\d+)?)", composition_string.split(")")[1])[1]
+        )
+        sub_composition_sum = sum(
+            [sub_composition[p] for p in sub_composition]
+        )
+        total_sum = composition_sum + sub_composition_percentage
+        scale_factor = 1
+        if total_sum > 1.0:
+            scale_factor = 100
+        sub_composition_percentage /= scale_factor
+        for element in sub_composition:
+            sub_composition[element] *= (
+                sub_composition_percentage / sub_composition_sum
+            )
+
         remaining_composition = parse_composition_string_block(
             remaining_composition_string
         )
+        for element in remaining_composition:
+            remaining_composition[element] *= 1 - sub_composition_percentage
 
         composition = {**sub_composition, **remaining_composition}
 
     else:
         composition = parse_composition_string_block(composition_string)
 
-    return filter_order_composition(composition)
+    return composition, structure
 
 
 def parse_composition_string_block(composition_string: str) -> dict:
@@ -1055,10 +1189,21 @@ def parse_composition_string_block(composition_string: str) -> dict:
 
     :group: alloy.utils
     """
+
     composition = {}
 
     split_composition = re.findall(r"[A-Z][^A-Z]*", composition_string)
 
+    composition_sum = 0
+    for element_percentage in split_composition:
+        split_element_percentage = re.split(
+            r"(\d+(?:\.\d+)?)", element_percentage
+        )
+        if len(split_element_percentage) > 1:
+            composition_sum += float(split_element_percentage[1])
+    if composition_sum == 0:
+        composition_sum = 1
+
     for element_percentage in split_composition:
         split_element_percentage = re.split(
             r"(\d+(?:\.\d+)?)", element_percentage
@@ -1072,7 +1217,8 @@ def parse_composition_string_block(composition_string: str) -> dict:
                 )
 
             composition[split_element_percentage[0]] = round(
-                float(split_element_percentage[1]) / 100.0, decimal_places
+                float(split_element_percentage[1]) / composition_sum,
+                decimal_places,
             )
         else:
             composition[split_element_percentage[0]] = 1.0 / len(
