@@ -7,7 +7,7 @@ def test_alloy_creation():
 
     alloy_from_dict = mg.Alloy({"Fe": 20, "Ag": 80})
 
-    assert alloy_from_string.__dict__ == alloy_from_dict.__dict__
+    assert alloy_from_string == alloy_from_dict
 
 
 def test_alloy_to_string():
@@ -59,3 +59,33 @@ def test_multiple_round():
     assert mg.alloy.multiple_round(0.015, 0.01) == 0.02
     assert mg.alloy.multiple_round(0.12345, 0.03) == 0.12
     assert mg.alloy.multiple_round(0.12345, 0.2) == 0.2
+
+
+def test_structure():
+
+    assert mg.Alloy("CuZr", "A1") == "Cu100[A1]"
+    assert mg.Alloy("CuZr", "B2") == "Cu50Zr50[B2]"
+    assert mg.Alloy("CuZr", "L1_0") == "Cu50Zr50[L1_0]"
+    assert mg.Alloy("ZrCu", "D2_1") == "Cu85.71Zr14.29[D2_1]"
+    assert mg.Alloy("CuZr", "D2_1") == "Zr85.71Cu14.29[D2_1]"
+
+
+def test_structure_change():
+
+    a = mg.Alloy("CuZr", "A1")
+    a.structure = "A2"
+    assert a == "Cu100[A2]"
+
+    a = mg.Alloy("CuZr", "B1")
+    a.structure = "A2"
+    assert a == "Cu100[A2]"
+
+    a = mg.Alloy("CuZr", "B1")
+    a.structure = "D0_23"
+    assert a == "Cu75Zr50[D0_23]"
+
+
+def test_scale_conversion():
+
+    assert mg.Alloy("Cu50Zr50") == mg.Alloy("Cu0.5Zr0.5")
+    assert mg.Alloy("Cu99.9Zr0.1") == mg.Alloy("Cu0.999Zr0.001")
